#!/bin/sh
#*******************************************************************************
# Copyright (c) 2016-2017 Hewlett-Packard Enterprise Development Company, LP.
# All rights reserved.
#*******************************************************************************
. /etc/cmcluster.conf

sgmgr_configure_jetty() {
    file_to_modify_list=""
    ibmJavaVersion=""
    sgmgr_install_loc=$1
    jetty_install_loc=$sgmgr_install_loc/jetty
    jetty_base_path=$sgmgr_install_loc/jetty-sgmgr-base
    
    
    while read line
    do
        if echo "$line" | egrep -q  "file-list-to-be-modified" ;then
            file_to_modify_list=`echo "$line" | awk -F= '{print $2}'`
        fi
        if echo  "$line" | egrep -q  "files-to-be-run" ;then
            executeFileList=`echo "$line" | awk -F= '{print $2}'`
        fi
    done < $sgmgr_install_loc/jetty-sgmgr.properties

    for file in ${file_to_modify_list}
    do
        tag=""
        while read line
        do
            if echo "$line" | egrep -q  "$file-tag" ;then
                tag=`echo "$line" | awk -F= '{print $2}'`
                sed -i '/<'$tag'/,$d' $jetty_install_loc/etc/$file.xml
                sed -n /$file-content-start/,/$file-content-end/p $sgmgr_install_loc/jetty-sgmgr.properties | \
                     grep -v "$file-content-start" | grep -v "$file-content-end" >> $jetty_install_loc/etc/$file.xml
            fi
        done < $sgmgr_install_loc/jetty-sgmgr.properties
    done	
    #copy the jaas.conf file from the SGMGR install location to jetty directory
    cp -f $sgmgr_install_loc/jaas.conf $jetty_install_loc/etc/jaas.conf
    
    # If IBM java is installed then change the protocol verstion to TLSv1.2
    ibmJavaVersion=`java -version 2>&1 | grep "IBM"`
    if [ ! -z "$ibmJavaVersion" ];then
        sed -i 's/<Set name="protocol">TLS<\/Set>/<Set name="protocol">TLSv1.2<\/Set>/' $jetty_install_loc/etc/jetty.xml
    fi

    # For Serviceguard core version that are less than 12.00.40 the ds.jar will not exist.
    # Remove the occurence of this jar file in jetty configuration.
    # do this only if it *not* debian distro.
    if ! isDebianDistro; then
        sgrootSedPattern=$(echo $SGROOT | sed -e 's/\//\\\//g')
    
        if [ ! -f $SGROOT/lib/serviceguard_ds.jar ];then
            sed -i 's/<Set name="extraClasspath">\%\%SGROOT\%\%\/lib\/serviceguard_ds.jar<\/Set>//g' $sgmgr_install_loc/contexts-sgmgr/sgmgr_rest.xml
        else
            # If there is a case where the Serviceguard manager is already upgraded but serviceguard is 
            # getting upgraded. In that case the pattern would have got removed as part of above if.
            # If the upgrade of Serviceguard manager and serviceguard core go hand in hand then
            # simple replace would suffice else we need to add this line in sgmgr_rest.xml.
                
            if ( grep -q 'serviceguard_ds.jar' $sgmgr_install_loc/contexts-sgmgr/sgmgr_rest.xml ) then
                sed -i 's/\%\%SGROOT\%\%/'"$sgrootSedPattern"'/g' $sgmgr_install_loc/contexts-sgmgr/sgmgr_rest.xml
            else
                # Add serviceguard_ds.jar in the sgmgr_rest.xml file.
                sed -i '/<\/Configure>/ i <Set name="extraClasspath">'"$sgrootSedPattern"'\/lib\/serviceguard_ds.jar<\/Set>' \
                                 $sgmgr_install_loc/contexts-sgmgr/sgmgr_rest.xml
            fi 
              
        fi
    fi
    
    # HACK for jetty.sh(same as jetty-sgmgr-start-stop.sh) to work when start-stop-daemon
    # command is installed on non-debian distros if the distro is SUSE then start-stop-daemon 
    # command does not work.
    # jetty.sh needs to change to not use start-stop-daemon command
    # the problem is observed in SUSE 11. Redhat does not ship start-stop-daemon
    # command. If someone installs this command on any distro jetty start will fail,
    # so fix it for non-debian distros in general.
    if ! isDebianDistro; then
         if ! $(grep -q 'false.*\&\&.*type start-stop-daemon' $jetty_install_loc/bin/jetty-sgmgr-start-stop.sh) ; then
             sed -i 's/\(type start-stop-daemon\)/false \&\& \1/g' $jetty_install_loc/bin/jetty-sgmgr-start-stop.sh
         fi
    fi

    #initialize the start of jetty
    $sgmgr_install_loc/jetty-startini.sh "$jetty_install_loc" "$jetty_base_path"
}

getOSDistroType() {
    /bin/cat /proc/version | /bin/grep -q "Red Hat"
    if [ $? -eq 0 ]; then
        echo "RedHat"
    fi
    /bin/cat /proc/version | /bin/grep -q "SUSE Linux"
    if [ $? -eq 0 ]; then
        echo "SUSE"
    fi 
    /bin/cat /proc/version | /bin/grep -q "Debian"
    if [ $? -eq 0 ]; then
        echo "DEBIAN"
    fi

}

log_msg() {
    echo "$*" 1>&2
}

isJettySystemctlRequired() {
    local suseVer=0
    osDistroType=$(getOSDistroType)
    if [ "$osDistroType" = "RedHat" ]; then
        #if redhat release is > 7 then systemctl is required.
        #for jetty this is not must. so returning 1.
        return 1
    fi
    if [ "$osDistroType" = "SUSE" ]; then
        if [ -f "/etc/SuSE-release" ]; then
            suseVer=`/bin/cat /etc/SuSE-release 2>/dev/null  | grep "VERSION =" | cut -d "=" -f 2 |  awk '{print $1}'`
            if [ $suseVer -ge 12 ];then
                #the version of SUSE is 12 then
                return 0
            fi
            return 1
        fi
        if [ -e "/etc/os-release" ]; then
            suseVer=`/bin/cat /etc/os-release 2>/dev/null | grep "VERSION_ID=" | awk -F "=\"" '{print $2'} | awk -F "." '{print $1}'`
            if [ $suseVer -ge 12 ]; then
                # the version of SUSE is 12 and greater
                return 0
            fi
        fi
        return 1
    fi
    if [ "$osDistroType" = "DEBIAN" ]; then
        return 0
    fi
}

isDebianDistro() {
    osDistroType=$(getOSDistroType)
    if [ "$osDistroType" = "DEBIAN" ]; then
        return 0
    fi
    return 1
}

isRHDistro() {
    osDistroType=$(getOSDistroType)
    if [ "$osDistroType" = "RedHat" ]; then
        return 0
    fi
    return 1
}
isSUSEDistro() {
    osDistroType=$(getOSDistroType)
    if [ "$osDistroType" = "SUSE" ]; then
        return 0
    fi
    return 1
}

isRH7orgreater() {
    local rhel_version=0
    osDistroType=$(getOSDistroType)
    if [ "$osDistroType" = "RedHat" ]; then
        if [ -f "/etc/redhat-release" ]; then
            /bin/cat /etc/redhat-release | grep -q "7\.[[:digit:]]*" 
            if [ $? -eq 0 ]; then
                return 0
            fi
        fi
        return 1
    fi
    return 1
}


getSgInstalledLoc() {
    echo $SGCONF
}

sgmgr_setup_config()
{
    local sgmgr_instloc="/opt/hp/cmcluster/serviceguardmanager"
    local jetty_base_path=""
    local jetty_install_location=""
    if isDebianDistro; then
        sgmgr_instloc="/opt/hpe/serviceguard/sgmgr"
        cp -f $sgmgr_instloc/debian/sgmgr_login /etc/pam.d/
    fi 
    if isSUSEDistro; then
        cp -f $sgmgr_instloc/suse/sgmgr_login /etc/pam.d/
    elif isRHDistro; then
        cp -f $sgmgr_instloc/redhat/sgmgr_login /etc/pam.d/
    fi
    sgmgr_configure_jetty $sgmgr_instloc
    if  ! $(grep -q "#start serviceguardmanager"  $SGCONF/cmclnodelist)  ; then
        echo "#start serviceguardmanager" >> $SGCONF/cmclnodelist
        echo "+ sgmgr" >> $SGCONF/cmclnodelist
        echo "#end serviceguardmanager" >> $SGCONF/cmclnodelist
    fi
    
    # Create jetty base path to avoid startup warnings
    jetty_install_location=$sgmgr_instloc/jetty
    jetty_base_path=$sgmgr_instloc/jetty-sgmgr-base
    mkdir -p $jetty_base_path
    if [ ! -f $jetty_base_path/etc ]; then
        ln -s -f $jetty_install_location/etc $jetty_base_path/etc
    fi
    if [ ! -f $jetty_base_path/start.ini ]; then
        ln -s -f $jetty_install_location/start.ini $jetty_base_path/start.ini
    fi
    if [ ! -f $jetty_base_path/start.jar ]; then
        ln -s -f $jetty_install_location/start.jar $jetty_base_path/start.jar
    fi
    return 0

}

# Caller of this function has to ensure that jetty instance is
# stopped before calling this function.
 
sgmgr_setup_erase()
{
    local removeUser=$1
    local eraseRetVal=0
    start="#start serviceguardmanager"
    end="#end serviceguardmanager"
    if isSUSEDistro; then
        if [ -e $SGCONF/cmclnodelist ]; then
            sed -i "/$start/,/$end/d" "$SGCONF/cmclnodelist" 
        fi
        if [ -e /etc/init.d/jetty-sgmgr ];then
            /sbin/insserv -r /etc/init.d/jetty-sgmgr
        fi
    elif isRHDistro; then
        if [ -e $SGCONF/cmclnodelist ]; then
            sed -i "/$start/,/$end/d" "$SGCONF/cmclnodelist"
        fi
        if [ -e /etc/init.d/jetty-sgmgr ];then
            /sbin/chkconfig --levels 35 jetty-sgmgr off 
        fi
    fi
    if ! isDebianDistro; then
        rm -f /etc/init.d/jetty-sgmgr
    fi
    if [ "$removeUser" = "y" ]; then
        sgmgr_remove_user || eraseRetVal="$?"
    fi
    if [ "$eraseRetVal" != "0" ]; then
        return $eraseRetVal
    fi
    return 0
}

sgmgr_add_user() {
    local username="sgmgr"
    local isSgmgrExist="false"
    local sgmgrEnv=$1
    local testUserAdd=0
    if [ $(id -u) -eq 0 ]; then 
        # check if user exists in the system.
        id sgmgr > /dev/null 2>&1 
        if [ $? -eq 0 ]; then
            isSgmgrExist="true"
        fi    

        if [ ! -z "$sgmgrEnv" ]; then 
            if [ "$isSgmgrExist" != "true" ]; then
                useradd $username > /dev/null 2>&1
                testUserAdd=$?
            fi
            if [ $testUserAdd -eq 0 ];then
                echo $username:$sgmgrEnv | chpasswd
                testUserAdd=$?
            fi

            if [ $testUserAdd -eq 0 ]; then 
                log_msg "INFO: User $username has been added/updated on the system!"
            else
                log_msg "INFO: Please add user $username before using Serviceguard Manager"
            fi
        else 
            if [ $isSgmgrExist != "true" ];then
                log_msg "ERROR: Could not create sgmgr user as the environment variable SGMGR_ENV is not set."
                log_msg "Set the environment variable SGMGR_ENV and rerun the setup script<$0 -c -u>."
                return 1 
            fi
        fi
    else 
        log_msg "ERROR:Only root can add a user to the system"
        return 1
    fi
    return 0
}

sgmgr_remove_user()
{
    local username="sgmgr"
    local userdelCmdArgs="-f"
    if [ $(id -u) -eq 0 ]; then
    {
        id sgmgr > /dev/null 2>&1 
        if [ $? -eq 0 ] ; then
            if [ -d /home/sgmgr ]; then
                userdelCmdArgs=$userdelCmdArgs" -r"
            fi
            userdel $userdelCmdArgs $username 
            log_msg "INFO: The user $username has been deleted"
        fi 
    }
    else
        log_msg "ERROR:Only root can remove a user to the system"
        return 1
    fi
    return 0
}
checkSGMgrUIForIBM()
{
    local sgmgr_install_loc=$1
    local gJettyStartWait=0
    local gJettyStartIncre=0
    i=0
    while read line
    do
        if echo  "$line" | egrep -q  "jetty-start-wait" ;then
            gJettyStartWait=`echo "$line" |  cut -f 2 -d '='`
        fi
        if echo "$line" | egrep -q  "jetty-start-increments" ;then
            gJettyStartIncre=`echo "$line" |  cut -f 2 -d '='`
        fi

    done < $sgmgr_install_loc/jetty-sgmgr.properties
    tJettyStartTime=0
    log_msg  " Please wait till jetty starts ."
    for i in $(seq 1 10) 
    do
        tJettyStartTime=$(($gJettyStartWait + $i * $gJettyStartIncre))
        sleep $tJettyStartTime
        vSGMgrUI=`java -cp $sgmgr_install_loc/SgmgrUtil.jar com.hp.SgmgrUITest  2>&1 | grep "Serviceguard Manager"`
        if [ "$vSGMgrUI" = "Serviceguard Manager" ];then
            break
        elif [ $i -eq 10 ]; then
            log_msg " "
            log_msg  "ERROR: Unable to start jetty instance."
            log_msg  "ERROR: Please rerun $sgmgr_install_loc/Serviceguard_manager_setup -c "
            break
        else
            if isJettySystemctlRequired; then
                systemctl daemon-reload
                systemctl start jetty-sgmgr
            else
                service jetty-sgmgr start >> $install_jetty_loc/logs/jetty_start_stop.log 2>&1 &
            fi
        fi
     done
}

updateMulticastPort()
{
    local lNodeSgVersion=0
    local sgmgr_install_loc=$1
    local mcastPortValue=$2
    local updateMportRetval=0
    lLocalNode=$(HOST=`hostname`; echo ${HOST%%.*})
    lcmviewcli=`${SGSBIN}/cmviewcl -f line -s config 2>/dev/null`

    #getting list of cluster nodes
    lNodeNames=`echo "$lcmviewcli" | awk -F "[:|=]" '(($1 == "node") && ($3 == "name")) {print $4}'`   
    #updating on  multicast port on local node 
    vSGMgrConfigl=` java -cp $sgmgr_install_loc/SgmgrUtil.jar com.hp.ConfigurationManager \
                             $sgmgr_install_loc/rest/Configuration.ser $mcastPortValue 2>&1 | \
                             grep "Configuration Setting Done"`
     
    if [ "$vSGMgrConfigl" = "Configuration Setting Done" ];then
        echo " Updated  Multicast Port : $mcastPortValue on $lLocalNode"
    fi     
    
    lTempDirSuf=`date +%d%m%Y%H%M%S`
    mkdir -p $sgmgr_install_loc/temp$lTempDirSuf
    touch $sgmgr_install_loc/temp$lTempDirSuf/remotejetty.log 
    for node in $lNodeNames
    do
        rm -rf $sgmgr_install_loc/temp$lTempDirSuf/*
        lWarningMsg="Warning: Failed to update multicast port to 5301 on $node.\
                     The multicast port must be changed to 5301 on $node before \
                     proceeding with any further operations. Log in to $node \
                     using Serviceguard Manager GUI to do the same."

        if [ "$node" != "$lLocalNode" ];then
        #copy setting file, jetty loc file and sg manger version files to local node
            ${SGSBIN}/cmcp $node:$sgmgr_install_loc/config/miscellaneous.properties \
                           $lLocalNode:$sgmgr_install_loc/temp$lTempDirSuf/ \
                           2>$sgmgr_install_loc/temp$lTempDirSuf/remotejetty.log
            if [ $? -ne 0 ];then
                echo "$lWarningMsg"
                updateMportRetval=10
                continue
            fi


            lRemoteFielcontent=`cat $sgmgr_install_loc/temp$lTempDirSuf/miscellaneous.properties \
                                    2>$sgmgr_install_loc/temp$lTempDirSuf/remotejetty.log ` 	
            if [ $? -ne 0 ];then
                echo "$lWarningMsg" 
                updateMportRetval=10
                continue
            fi
            lNodeSgVersion=`echo "$lRemoteFielcontent" | awk -F. '/^Serviceguard_Manager_Build_version/{print $4}'`
            if [ $lNodeSgVersion -lt 50 ];then
                ${SGSBIN}/cmcp $node:/usr/local/rest/Configuration.ser \
                               $node:$sgmgr_install_loc/jetty_location.txt \
                               $lLocalNode:$sgmgr_install_loc/temp$lTempDirSuf/ \
                               2>$sgmgr_install_loc/temp$lTempDirSuf/remotejetty.log
                if [ $? -ne 0 ];then
                    echo "$lWarningMsg"
                    updateMportRetval=10
                    continue
                fi
                vSGMgrConfig=`java -cp $sgmgr_install_loc/SgmgrUtil.jar com.hp.ConfigurationManager \
                               $sgmgr_install_loc/temp$lTempDirSuf/Configuration.ser $mcastPortValue \
                               2>&1 | grep "Configuration Setting Done"`
    
                if [ "$vSGMgrConfig" = "Configuration Setting Done" ];then
                    echo " Updating Multicast Port : $mcastPortValue on $node"
                    ${SGSBIN}/cmcp $lLocalNode:$sgmgr_install_loc/temp$lTempDirSuf/Configuration.ser  $node:/usr/local/rest/ \
                                   2>$sgmgr_install_loc/temp$lTempDirSuf/remotejetty.log
                    if [ $? -ne 0 ];then
                        echo "$lWarningMsg"
                        setExitWithWarning
                        continue
                    fi
    
                    # checking for redhat or dayone release on node
                    if isRHDistro || [ $lNodeSgVersion -lt 10 ]; then
                        echo "Restarting  jetty-sgmgr on  $node node "
                        ${SGSBIN}/cmexec $node -o $sgmgr_install_loc/temp$lTempDirSuf/remotejetty.log service jetty-sgmgr restart \
                                                     2>$sgmgr_install_loc/temp$lTempDirSuf/remotejetty.log
                        if [ $? -ne 0 ];then
                            echo "$lWarningMsg"
                            updateMportRetval=10
                            continue
                        fi
                     else
                         echo "Restarting  jetty-sgmgr on  $node node using Serviceguard_manager_setup"
                         lJettyLocalcontent=`cat $sgmgr_install_loc/temp$lTempDirSuf/jetty_location.txt`
                         lNodeJettyLoc=`echo "$lJettyLocalcontent" | awk -F= '/^JETTY_HOME/{print $2}'`
                         ${SGSBIN}/cmexec $node -o $sgmgr_install_loc/temp$lTempDirSuf/remotejetty.log \
                                                   $sgmgr_install_loc/Serviceguard_manager_setup \
                                                   -l $lNodeJettyLoc -o config  2 >$sgmgr_install_loc/temp$lTempDirSuf/remotejetty.log
                         if [ $? -ne 0 ];then
                             echo "$lWarningMsg"
                             updateMportRetval=10
                             continue
                         fi
                     fi
                fi
            fi
        fi
    done
    rm -Rf $sgmgr_install_loc/temp$lTempDirSuf 
    return $updateMportRetval
}
